//!MENU-ENTRY: Convert AW Patches

log("AW Patch convertion started");

Promise.all(
    dirRec(path.dirname("."))
    .filter( fileName => /\.awpatch$/i.test(fileName) )
    .map( patchName => savePatch(patchName.replace(/\.awpatch/i, ".h"), JSON.parse(read(patchName))) )
).then(_=>{
    log("AW Patch convertion finished");
    if(hookTrigger == "pre-build") hookArgs[1]();
}).catch(_=>{
    if(hookTrigger == "pre-build") hookArgs[1]("Conversion error");
});

function dirRec(name){
    let out = [];
    dir(name).forEach(child=>{
        const fullChild = path.join(name, child);
        out = out.concat(stat(fullChild).isDirectory() ? dirRec(fullChild) : [fullChild]);
    });
    return out;
}

function savePatch(filename, patch) {
    const waveforms = {
        square:   "return Audio::AWSynthSource::sqr(p);",
        sine:     "return Audio::AWSynthSource::sin(p);",
        triangle: "return Audio::AWSynthSource::tri(p);",
        sawtooth: "return Audio::AWSynthSource::saw(p);",
    };
    
    let name = filename.substr(0, filename.lastIndexOf("."));   // Remove filename extension
    name = name.replace( /\W/g , '_');                          // Replace every non alphanumeric character with '_'
    if(/^[0-9]./.test(name)) name = "_"+name;                   // Insert '_' if name starts with number
    
    let out = "#pragma once\n\n";
    out += "#include \"AWSynthSource.h\"\n\n";
    out += "constexpr auto "+path.basename(name)+" = Audio::AWPatch([](std::uint32_t t, std::uint32_t p)->std::int32_t { ";
    out += waveforms[patch.waveform] + " })\n";
    out += "    .volume("+patch.volume+").step("+patch.step+").release("+patch.release+").glide("+patch.glide+")\n";
    out += "    .semitones(Audio::AWPatch::Envelope(";
    out += patch.semitones.data.join();
    out += ").smooth("+patch.semitones.smooth+").loop("+patch.semitones.loop_start+","+patch.semitones.length+"))\n";
    out += "    .amplitudes(Audio::AWPatch::Envelope(";
    out += patch.amplitudes.data.join();
    out += ").smooth("+patch.amplitudes.smooth+").loop("+patch.amplitudes.loop_start+","+patch.amplitudes.length+"));\n";
    
    write(filename, out, undefined);
    return true;
}

